<?php
namespace Controller;

use React\Stream\Util;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Service;
use Util\Messages;

class Issue extends \Generated\Controller\Issue
{
    /**
     * @var Service\Task $taskService
     */
    private $taskService;

    /**
     * @var Service\Issue $issueService
     */
    private $issueService;


    public function __construct(Manager $dataSourceManager, Service\Task $taskService, Service\Issue $issueService, Utilities $utilities){
        parent::__construct($dataSourceManager);
        $this->taskService = $taskService;
        $this->issueService = $issueService;
        $this->utilities = $utilities;
    }

    public function findLog(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $issueId    = Utilities::getValueInFilterDataSourceByName($conditions, 'issueId', true);
        $filter = Utilities::createQueryFiltered($conditions);
        try {
            $dataSet = $this->taskService->findIssueLogByIssueId($issueId[0], $filter);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch(\Exception $exception){
            $response->addMessage(new Response\Message(Messages::ERR_CANT_GET_ISSUE_LOGS_EXPLANATION . $exception->getMessage()));
        }
    }

    public function itemBudget(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $issueId    = Utilities::getValueInFilterDataSourceByName($conditions, 'issueId', true);
        $filter = Utilities::createQueryFiltered($conditions);
        try {
            $dataSet = InstanceProvider::getService()->getComment()->itemBudget($issueId[0], $filter);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findBoardIssues(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $boardId = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'BOARD_ID',true);
            $structureIds = Utilities::getValueInFilterDataSourceByName($filter, 'STRUCTURE', true);
            $projectStatusIds = Utilities::getValueInFilterDataSourceByName($filter, 'PROJECT_STATUS_ID', true);
            Utilities::getValueInFilterDataSourceByName($filter, 'PRODUCT', true);
            $issues = $this->taskService->findBoardIssues($boardId, $filter, $structureIds, $projectStatusIds);
            $response->addDataSet(new DataSource\DataSet('findBoardIssues', $issues));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function findIssueOrderToFrame(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $issueId = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE',true);
            $issues = $this->issueService->findIssueOrderToFrame($issueId);
            $response->addDataSet(new DataSource\DataSet('issueOrderToFrame', $issues));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function maintenanceCost(Request\Filter $request, Response $response) {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $issueId = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'ISSUE');

            $budget = InstanceProvider::getService()->getBudget()->findMaintenanceCost($issueId);
             $response->addDataSet(new DataSource\DataSet('MaintenanceCost', [$budget]));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveMaintenanceCost(Request\DataSet $request, Response $response) {
        try{
            $rows = $request->getDataSet()->getRows()[0];

            $budget = InstanceProvider::getService()->getBudget()->saveMaintenanceCost($rows);
             $response->addDataSet(new DataSource\DataSet('MaintenanceCost', [$budget]));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }



    public function findCommentTypeIssueStatus(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $issues = $this->issueService->findCommentTypeIssueStatus($conditions);
            DevOpsUtilities::setDataSetDefault($request,$response,$issues);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findEntranceCommentIssueStatus(Request\Filter $request, Response $response){
        try {

            $filter = $request->getFilterCriteria()->getConditions();
            $issueStatus = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE_STATUS',true);
            $issues = $this->issueService->findEntranceCommentIssueStatus($issueStatus);
            DevOpsUtilities::setDataSetDefault($request,$response,$issues);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findOutputCommentIssueStatus(Request\Filter $request, Response $response){
        try {

            $filter = $request->getFilterCriteria()->getConditions();
            $issueStatus = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE_STATUS',true);
            $issues = $this->issueService->findOutputCommentIssueStatus($issueStatus);
            DevOpsUtilities::setDataSetDefault($request,$response,$issues);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findIssueFramingClassification(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $issueId = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE',true);
            $issues = $this->issueService->findIssueFramingClassification($issueId);
            DevOpsUtilities::setDataSetDefault($request,$response,$issues);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findIssueFramingProjectType(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $issueId = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE',true);
            $issues = $this->issueService->findIssueFramingProjectType($issueId);
            DevOpsUtilities::setDataSetDefault($request,$response,$issues);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeIssueFramingProjectType(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $this->issueService->removeIssueFramingProjectType( $row['ID']);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function taskClassificationByIssue(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $issueId = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE_ID');
            $classifications = $this->issueService->taskClassificationByIssue($issueId);
            DevOpsUtilities::setDataSetDefault($request,$response,$classifications);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function taskClassificationByIssues(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $issueId = DevOpsUtilities::getValueInFilterDataSourceByName($filter,'ISSUE_IDS');
            $classifications = $this->issueService->taskClassificationByIssues($issueId);
            DevOpsUtilities::setDataSetDefault($request,$response,$classifications);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveAddTaskClassification(Request\DataSet $request, Response $response){
        $issueClassifications = $request->getDataSet()->getRows();
        try {
            $taskTypeId = $issueClassifications[0]['taskType'];
            $this->issueService->saveAddTaskClassification($issueClassifications,$taskTypeId);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveIssuesPriority(Request\DataSet $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        try {
            $issues =  $row['issues'];
            $priority =  empty($row['priority']) ? null : $row['priority']; ;
            $this->issueService->saveIssuesPriority($priority,$issues);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveAddProjectType(Request\DataSet $request, Response $response){
        $projectTypes = $request->getDataSet()->getRows();
        try {
            $taskTypeId = $projectTypes[0]['taskType'];
            $this->issueService->saveAddProjectType($projectTypes,$taskTypeId);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeCurrentUserFollower(Request\DataSet $request, Response $response){
        $rows = $request->getDataSet()->getRows();
        try {
            $issueId = $rows[0]['ISSUE_ID'];
            $this->issueService->removeCurrentUserFollower($issueId);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED_FOLLOWER));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveOutputCommentIssueStatus(Request\DataSet $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        try {
            $action = $row['ACTION_LABEL'];
            $issueClassification = $row['ISSUE_CLASSIFICATION'];
            $description = $row['DESCRIPTION'];
            $commentType = $row['COMMENT_TYPE'];
            $targetStatus = $row['TARGET_STATUS'];
            $taskType = $row['TASK_TYPE'];
            $outputCommentId = empty($row['ID']) ? 0 : $row['ID'];
            $taskTypeCommentType = $this->issueService->saveOutputCommentIssueStatus($outputCommentId,$action,$issueClassification,$description,
                $commentType,$targetStatus,$taskType);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
            $response->addDataSet(new DataSource\DataSet(explode('/',$request->getRoutePath())[1], array('ID' => $taskTypeCommentType->getId())));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveSendOutComment(Request\DataSet $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        try {
            $description = $row['DESCRIPTION'];
            $document =  empty($row['DOCUMENT']) ? array() : $row['DOCUMENT']; 
            $taskTypeTarget = empty($row['TASK_TYPE_TARGET']) ? array() : $row['TASK_TYPE_TARGET'];
            $commentTypeId = $row['COMMENT_TYPE_ID'];
            $issueId = $row['ISSUE_ID'];
            $this->issueService->sendOutComment($issueId, $document, $taskTypeTarget, $commentTypeId, $description);
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveEntranceCommentIssueStatus(Request\DataSet $request, Response $response){
        $rows = $request->getDataSet()->getRows()[0];
        try {
            $commentType = $rows['COMMENT_TYPE'];
            $description = $rows['DESCRIPTION'];
            $issueClassificationId = $rows['ISSUE_CLASSIFICATION'];
            $taskTypeId = $rows['TASK_TYPE'];
            $this->issueService->saveEntranceCommentIssueStatus($taskTypeId,$issueClassificationId,$description,$commentType);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeIssueFramingClassification(Request\Row $request, Response $response){
        $taskTypeClassification = $request->getRow();
        try {
            $taskTypeClassificationId = $taskTypeClassification['ID'];
            $this->issueService->removeTaskTypeClassification($taskTypeClassificationId);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeOutputCommentIssueStatus(Request\Row $request, Response $response){
        $row = $request->getRow();
        try {
            $id = $row['ID'];
            $this->issueService->removeOutputCommentIssueStatus($id);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveIssueOrderToFrame(Request\DataSet $request, Response $response) {
        $issue = $request->getDataSet()->getRows()[0];
        try {

            $orderToFrame = $issue['ORDER_TO_FRAME'];
            $taskTypeId = $issue['ID'];
            $allowPlayTaskInFrame = $issue['ALLOW_PLAY_TASK_IN_FRAME'];

            $this->issueService->saveIssueOrderToFrame($taskTypeId,$orderToFrame,$allowPlayTaskInFrame);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function markAsRead(Request\Filter $filter, Response $response){
        $conditions = $filter->getFilterCriteria()->getConditions();
        $issuesIds  = $conditions[0]['value'];
        try {
            $this->issueService->markManyAsRead($issuesIds);
            $response->addDataSet(new DataSource\DataSet('isLastPostViewed', array(true)));
        } catch(\Exception $exception){
            $response->addMessage(new Response\Message($exception->getMessage()));
        }
    }

    public function validateEntranceComment(Request\Filter $filter, Response $response){
        try {
            $conditions = $filter->getFilterCriteria()->getConditions();
            $issueId = Utilities::getValueInFilterDataSourceByName($conditions,'ISSUE_ID');
            $typeTask = Utilities::getValueInFilterDataSourceByName($conditions,'TYPE_TASK');
            $dataset = $this->issueService->validateEntranceComment($issueId,$typeTask);

            DevopsUtilities::setDataSetDefault($filter, $response, $dataset);
        } catch(\Exception $exception){
            $response->addMessage(new Response\Message($exception->getMessage()));
        }
    }

    public function validateSpecificationRequirements(Request $request, Response $response){
        try {
            $dataSet = $this->issueService->validateSpecificationRequirements();
            $response->addDataSet(new DataSource\DataSet('validateSpecificationRequirements', $dataSet));
        } catch(\Exception $exception){
            $response->addMessage(new Response\Message($exception->getMessage()));
        }
    }

    public function validateFinishSpecificationTask(Request\DataSet $request, Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $dataSet = $this->issueService->validateFinishSpecificationTask($row['TASK_ID']);
            $response->addDataSet(new DataSource\DataSet('validateFinishSpecificationTask', $dataSet));
        } catch(\Exception $exception){
            $response->setError(new Response\Error($exception->getMessage(),$exception->getCode()));
        }
    }

    public function clientAccept(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $description = $row['DESCRIPTION'];
            $document = $row['DOCUMENT'];
            $commentTypeId = $row['COMMENT_TYPE_ID'];
            $issueId = $row['ISSUE_ID'];
            $this->issueService->clientAccept($issueId,$commentTypeId,$description,$document);
            $response->addNotification(new Response\Notification(Messages::SUC_SPECIFICATION_ACCEPT));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro : '.$e->getMessage(),Response\Message::TYPE_ERROR));
        }
    }
    public function clientRefuse(Request\DataSet $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $description = $row['DESCRIPTION'];
            $document = $row['DOCUMENT'];
            $commentTypeId = $row['COMMENT_TYPE_ID'];
            $issueId = $row['ISSUE_ID'];
            $this->issueService->clientRefuse($issueId,$commentTypeId,$description,$document);
            $response->addNotification(new Response\Notification(Messages::SUC_SPECIFICATION_REFUSE));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro : '.$e->getMessage(),Response\Message::TYPE_ERROR));
        }
    }

    public function save(Request $request, Response $response, $showMessage = true) {
        $issue = $request->getDataSet()->getRows();
        $issue = $issue[0];
        try {
            list($board, $reporter, $dueDate, $endDate, $priority, $percentCompleted, $description, $followers,
                $labels, $sponsor, $labelsNotEditable, $isRelevant, $issueId, $issueLabel, $issueIssueClassification,
                $issueIssueStatus, $productVersionId, $version, $projectId, $isBoardIssue, $productVersionDevId, $issueResponse, $cditemtrab) = $this->buildIssue($issue);
            $dataSet = $this->taskService->saveIssue(
                $issueId, $issueLabel, $issueIssueClassification,
                $issueIssueStatus, $reporter, $dueDate, $priority,
                $percentCompleted, $description, $followers, $endDate, $board,
                $labels, $sponsor, $labelsNotEditable, $isRelevant, $productVersionId, $version, $projectId,
                $isBoardIssue, $productVersionDevId, $issueResponse, $cditemtrab
            );

            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            $response->addDataSet(new DataSource\DataSet('issue', [$dataSet]));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage()));
        }
    }
    public function saveAccompanyingIssue(Request\DataSet $request, Response $response) {
        $issue = $request->getDataSet()->getRows();
        $issue = $issue[0];
        try {
            $labels     = is_array($issue["LABELS"]) ?  $issue["LABELS"] : array();
            $labelsNotEditable = empty($issue['LABELS_NOT_EDITABLE']) ? array() : $issue['LABELS_NOT_EDITABLE'];
            $issueId = $issue['ID'];

            $this->taskService->setLabelAccompanyingIssue($issueId,$labels, $labelsNotEditable);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage()));
        }
    }

    public function delete(Request $request,Response $response,$showMessage = true){
        $issue = $request->getRow();
        try{
            $this->taskService->deleteIssues($issue['ID']);
            $response->addNotification(new Response\Notification(Messages::SUC_ISSUE_SUCCESSFUL_DELETED));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function associateIssuesAndLabels(Request\DataSet $request, Response $response){
        $issuesAndLabels = $request->getDataSet()->getRows()[0];
        $issues = $issuesAndLabels['issues'];
        $labels = $issuesAndLabels['labels'];
        try {
            $this->issueService->associateIssuesAndLabels($issues, $labels);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function saveIssueRelationship(Request\DataSet $request, Response $response)
    {
        $tasks = $request->getDataSet()->getRows();
        $count = 0;
        foreach ($tasks as $task) {
            try {
                $taskId = $task['ID'];
                $issueIds = $task['ISSUE'];
                $this->taskService->onlyCreateTaskRelationships($taskId, $issueIds, Entities\TaskRelationship::ISSUE_WITH_ISSUE_TYPE);
                $count++;
            } catch (\Exception $e) {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
        }
        $response->addMessage(new Response\Message(count($tasks[0]['ISSUE']) . ' issue(s) relacionada(s) a ' . $count . ' issue(s)'));
    }


    public function findIssueRelationship(Request\Filter $request, Response $response)
    {
            $dataSet = $request->getFilterCriteria()->getConditions();
            $taskId = Utilities::getValueInFilterDataSourceByName($dataSet, 'issue', true);
            $filter = Utilities::createQueryFiltered($dataSet);
            try {
                $task = $this->issueService->getIssueRelationship($taskId[0], $filter);
                DevopsUtilities::setDataSetDefault($request, $response, $task);
            } catch (\Exception $e) {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
    }

    public function getBudgetProduct(Request\Filter $request, Response $response)
    {
            $dataSet = $request->getFilterCriteria()->getConditions();
            $issueId = Utilities::getValueInFilterDataSourceByName($dataSet, 'ISSUE_ID', true);
            try {
                $task = $this->issueService->getBudgetProduct($issueId);

               $response->addDataSet(new DataSource\DataSet('issue', [$task]));
            } catch (\Exception $e) {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
    }

    public function getBudgetProductByTask(Request\Filter $request, Response $response)
    {
            $dataSet = $request->getFilterCriteria()->getConditions();
            $taskId = Utilities::getValueInFilterDataSourceByName($dataSet, 'TASK_ID', true);

            try {
                $task = $this->issueService->getBudgetProductByTask($taskId);

               $response->addDataSet(new DataSource\DataSet('issue', [$task]));
            } catch (\Exception $e) {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
    }

    public function findAllIssueInPagination(Request\Filter $request, Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", DataSource\FilterCriteria::EQ, $this->utilities->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    public function removeIssueRelationship(Request\DataSet $request, Response $response)
    {
        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as $issueRelationship) {
            try {
                $this->taskService->deleteTaskRelationshipByType($issueRelationship['ISSUE'], $issueRelationship['ID'],
                    Entities\TaskRelationship::ISSUE_WITH_ISSUE_TYPE);
            } catch (\Exception $e) {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
        }
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_DEL_ISSUE_WITH_ISSUE_TYPE));
    }

    public function saveProductBudget(Request\DataSet $request, Response $response)
    {
        $dataSet = $request->getDataSet()->getRows()[0];
        $productId = $dataSet['PRODUCT_ID'];
        $issueId = $dataSet['ISSUE_ID'];
        $cpCall = $this->issueService->saveProductBudget($productId, $issueId);
        $response->addNotification(new Response\Notification(Messages::SUC_SAVED_PRODUCT_BUDGET));
    }

    public function saveInfoIssueDependent(Request\DataSet $request, Response $response)
    {
        $issue = $request->getDataSet()->getRows();
        $param = $issue[0];
        try {
            list($name, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted, $description,
                $board, $followers, $labels,  $sponsor,$issueDependentId, $endDate) = $this->buildNewIssue($param);
            $this->taskService->saveIssueAndRelationship(
                $name, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted, $description,
                $board, $followers, $sponsor, $labels, $issueDependentId, $endDate
            );

            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param $issue
     * @return array
     */
    private function buildIssue($issue)
    {
        $board = empty($issue["BOARD"]) ? null : $issue["BOARD"];
        $reporter = empty($issue['REPORTER']) ? null : $issue['REPORTER'];
        $dueDate = empty($issue["DUE_DATE"]) ? "" : $issue["DUE_DATE"];
        $endDate = empty($issue["END_DATE"]) ? "" : $issue["END_DATE"];
        $priority = empty($issue["PRIORITY"]) ? null : $issue["PRIORITY"];
        $percentCompleted = empty($issue["PERCENT_COMPLETED"]) ? 0 : $issue["PERCENT_COMPLETED"];
        $description = empty($issue["DESCRIPTION"]) ? "" : $issue["DESCRIPTION"];
        $followers = empty($issue["FOLLOWERS"]) ? array() : $issue["FOLLOWERS"];
        $labels = empty($issue["LABELS"]) ? array() : $issue["LABELS"] ;
        $sponsor = empty($issue['SPONSOR_ID']) ? null : $issue['SPONSOR_ID'];
        $labelsNotEditable = empty($issue['LABELS_NOT_EDITABLE']) ? array() : $issue['LABELS_NOT_EDITABLE'];
        $isRelevant = empty($issue['IS_RELEVANT']) ? Service\Task::NOT_RELEVANT : $issue['IS_RELEVANT'];
        $issueId = $issue['ID'];
        $issueLabel = $issue['LABEL'];
        $issueIssueClassification = $issue['ISSUE_CLASSIFICATION'];
        $issueIssueStatus = $issue['ISSUE_STATUS'];
        $productVersionId = empty($issue["PRODUCT_VERSION_ID"]) ? null : $issue["PRODUCT_VERSION_ID"];
        $version = empty($issue["VERSION"]) ? null : $issue["VERSION"];
        $projectId = empty($issue["PRODUCT"]) ? null : $issue["PRODUCT"];
        $isBoardIssue = empty($issue["IS_BOARD_ISSUE"]) ? null : $issue["IS_BOARD_ISSUE"];
        $productVersionDevId = empty($issue["PRODUCT_VERSION_DEV_ID"]) ? null : $issue["PRODUCT_VERSION_DEV_ID"];
        $issueResponse = empty($issue["RESPONSE"]) ? null : $issue["RESPONSE"];
        $cditemtrab = empty($issue["CDITEMTRAB"]) ? null : $issue["CDITEMTRAB"];
        return array($board, $reporter, $dueDate, $endDate, $priority, $percentCompleted, $description, $followers, $labels, $sponsor, $labelsNotEditable, $isRelevant, $issueId, $issueLabel, $issueIssueClassification, $issueIssueStatus, $productVersionId, $version, $projectId, $isBoardIssue, $productVersionDevId, $issueResponse, $cditemtrab);
    }

    /**
     * @param $param
     * @return array
     */
    private function buildNewIssue($param)
    {
        $name = $param["LABEL"];
        $classification = $param["ISSUE_CLASSIFICATION"];
        $issueStatus = $param["ISSUE_STATUS"];
        $reporter = empty($param["REPORTER"]) ? null : $param["REPORTER"];
        $dueDate = empty($param["DUE_DATE"]) ? null : $param["DUE_DATE"];
        $priority = empty($param["PRIORITY"]) ? null : $param["PRIORITY"];
        $percentCompleted = empty($param["PERCENT_COMPLETED"]) ? 0 : $param["PERCENT_COMPLETED"];
        $description = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $board = $param["BOARD"];
        $followers = empty($param["FOLLOWERS"]) ? array() : $param['FOLLOWERS'];
        $sponsor = empty($param['SPONSOR_ID']) ? null : $param['SPONSOR_ID'];
        if (isset($param["LABELS"])) {
            $labels = is_array($param["LABELS"]) ? $param["LABELS"] : null;
        } else {
            $labels = null;
        }
        $issueDependentId = empty($param["ISSUE"]) ? null : $param["ISSUE"];
        $endDate = empty($issue["END_DATE"]) ? null : $issue["END_DATE"];
        return array($name, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted, $description, $board, $followers, $sponsor, $labels, $issueDependentId, $endDate);
    }


}
