<?php

namespace Service\Calendar\Restriction\Strategy;


use Service\Calendar\TaskCalendar;
use Service\Calendar\TaskCalendarInterface;

class RestrictionFactory {

    CONST SHOULD_START = 'DI';
    CONST SHOULD_END = 'DT';
    CONST DO_NOT_START_BEFORE = 'IA';
    CONST DO_NOT_START_AFTER = 'ID';
    CONST DO_NOT_END_BEFORE = 'TA';
    CONST DO_NOT_END_AFTER = 'TD';
    CONST AS_SOON_AS_POSSIBLE = 'MB';
    CONST AS_LATE_AS_POSSIBLE = 'MT';
    public static $ARRAY_RESTRICTION = array(
        self::SHOULD_START => "Deve Iniciar em",
        self::SHOULD_END => "Deve Terminar em",
        self::DO_NOT_START_BEFORE => "Não Iniciar Antes de",
        self::DO_NOT_START_AFTER => "Não Iniciar Depois de",
        self::DO_NOT_END_BEFORE => "Não Terminar Antes de",
        self::DO_NOT_END_AFTER => "Não Terminar Depois de",
        self::AS_SOON_AS_POSSIBLE => "O Mais Breve Possível",
        self::AS_LATE_AS_POSSIBLE => "O Mais Tarde Possível",
    );
    public static $ARRAY_RESTRICTION_PARENT = array(
        self::DO_NOT_START_BEFORE ,
        self::DO_NOT_END_AFTER,
        self::AS_SOON_AS_POSSIBLE ,
    );


    /**
     * @param TaskCalendarInterface $task
     * @return DoNotEndAfter|DoNotEndBefore|DoNotStartAfter|DoNotStartBefore|ShouldEnd|ShouldStart
     */
    public static  function factory($task, TaskCalendar $taskCalendar){
        switch ($task->getDateExceptionType()) {
            case self::SHOULD_START :
                $strategy = new ShouldStart($task, $taskCalendar);
                break;
            case self::SHOULD_END :
                $strategy = new ShouldEnd($task, $taskCalendar);
                break;
            case self::DO_NOT_START_BEFORE :
                $strategy = new DoNotStartBefore($task, $taskCalendar);
                break;
            case self::DO_NOT_START_AFTER :
                $strategy = new DoNotStartAfter($task, $taskCalendar);
                break;
            case self::DO_NOT_END_BEFORE :
                $strategy = new DoNotEndBefore($task, $taskCalendar);
                break;
            case self::DO_NOT_END_AFTER :
                $strategy = new DoNotEndAfter($task, $taskCalendar);
                break;
            case self::AS_LATE_AS_POSSIBLE :
                $strategy = new AsLateAsPossible($task, $taskCalendar);
                break;
            default :
                $strategy = new AsSoonAsPossible($task, $taskCalendar);
                break;
        }
        return $strategy;
    }


    public function getDatesByPrecursor($beginTask)
    {
    }
}